@@ -34,6 +34,7 @@
 #include "mount.h"
 #include "error.h"
 #include "network.h"
+#include "parse_opt.h"
 #include "parse_dev.h"
 
 #if !defined(MNT_FORCE)
@@ -133,6 +134,64 @@ static int del_mtab(const char *spec, const char *node)
 	return EX_FILEIO;
 }
 
+/*
+ * Discover mount server's hostname/address by examining mount options
+ *
+ * Returns a pointer to a string that the caller must free, on
+ * success; otherwise NULL is returned.
+ */
+static char *nfs_umount_hostname(struct mount_options *options,
+				 char *hostname)
+{
+	char *option;
+
+	option = po_get(options, "mountaddr");
+	if (option)
+		goto out;
+	option = po_get(options, "mounthost");
+	if (option)
+		goto out;
+	option = po_get(options, "addr");
+	if (option)
+		goto out;
+
+	return hostname;
+
+out:
+	free(hostname);
+	return strdup(option);
+}
+
+/*
+ * Returns EX_SUCCESS if mount options and device name have been
+ * parsed successfully; otherwise EX_FAIL.
+ */
+static int nfs_umount_do_umnt(struct mount_options *options,
+			      char **hostname, char **dirname)
+{
+	struct sockaddr_storage address;
+	struct sockaddr *sap = (struct sockaddr *)&address;
+	socklen_t salen = sizeof(address);
+	struct pmap nfs_pmap, mnt_pmap;
+
+	nfs_options2pmap(options, &nfs_pmap, &mnt_pmap);
+
+	*hostname = nfs_umount_hostname(options, *hostname);
+	if (!*hostname) {
+		nfs_error(_("%s: out of memory"), progname);
+		return EX_FAIL;
+	}
+
+	if (nfs_name_to_address(*hostname, AF_UNSPEC, sap, &salen)) {
+		if (nfs_advise_umount(sap, salen, &mnt_pmap, dirname) != 0)
+			return EX_SUCCESS;
+		else
+			nfs_error(_("%s: Server failed to unmount '%s:%s'"),
+					progname, *hostname, *dirname);
+	}
+	return EX_FAIL;
+}
+
 /*
  * Pick up certain mount options used during the original mount
  * from /etc/mtab.  The basics include the server's IP address and
@@ -142,85 +201,28 @@ static int del_mtab(const char *spec, const char *node)
  * version, and transport protocol used to punch through a firewall.
  * We will need this information to get through the firewall again
  * to do the umount.
+ *
+ * Note that option parsing failures won't necessarily cause the
+ * umount request to fail.  Those values will be left zero in the
+ * pmap tuple.  If the GETPORT call later fails to disambiguate them,
+ * then we fail.
  */
-static int do_nfs_umount23(const char *spec, char *opts)
+static int nfs_umount23(const char *devname, char *string)
 {
-	char *hostname;
-	char *dirname;
-	clnt_addr_t mnt_server = { &hostname, };
-	struct mntent mnt = { .mnt_opts = opts };
-	struct pmap *pmap = &mnt_server.pmap;
-	char *p;
-	int result = EX_USAGE;
+	char *hostname, *dirname;
+	struct mount_options *options;
+	int result = EX_FAIL;
 
-	if (!nfs_parse_devname(spec, &hostname, &dirname))
-		return result;
+	if (!nfs_parse_devname(devname, &hostname, &dirname))
+		return EX_USAGE;
 
-#ifdef NFS_MOUNT_DEBUG
-	printf(_("host: %s, directory: %s\n"), hostname, dirname);
-#endif
+	options = po_split(string);
+	if (options) {
+		result = nfs_umount_do_umnt(options, &hostname, &dirname);
+		po_destroy(options);
+	} else
+		nfs_error(_("%s: option parsing error"), progname);
 
-	if (opts && (p = strstr(opts, "addr="))) {
-		char *q;
-
-		free(hostname);
-		p += 5;
-		q = p;
-		while (*q && *q != ',') q++;
-		hostname = xstrndup(p,q-p);
-	}
-
-	if (opts && (p = strstr(opts, "mounthost="))) {
-		char *q;
-
-		free(hostname);
-		p += 10;
-		q = p;
-		while (*q && *q != ',') q++;
-		hostname = xstrndup(p,q-p);
-	}
-
-	pmap->pm_prog = MOUNTPROG;
-	pmap->pm_vers = 0; /* unknown */
-	if (opts && (p = strstr(opts, "mountprog=")) && isdigit(*(p+10)))
-		pmap->pm_prog = atoi(p+10);
-	if (opts && (p = strstr(opts, "mountport=")) && isdigit(*(p+10)))
-		pmap->pm_port = atoi(p+10);
-	if (opts && hasmntopt(&mnt, "v2"))
-		pmap->pm_vers = nfsvers_to_mnt(2);
-	if (opts && hasmntopt(&mnt, "v3"))
-		pmap->pm_vers = nfsvers_to_mnt(3);
-	if (opts && (p = strstr(opts, "vers=")) && isdigit(*(p+5)))
-		pmap->pm_vers = nfsvers_to_mnt(atoi(p+5));
-	if (opts && (p = strstr(opts, "mountvers=")) && isdigit(*(p+10)))
-		pmap->pm_vers = atoi(p+10);
-	if (opts && (hasmntopt(&mnt, "udp")
-		     || hasmntopt(&mnt, "proto=udp")
-		     || hasmntopt(&mnt, "mountproto=udp")
-		    ))
-		pmap->pm_prot = IPPROTO_UDP;
-	if (opts && (hasmntopt(&mnt, "tcp")
-		     || hasmntopt(&mnt, "proto=tcp")
-		     || hasmntopt(&mnt, "mountproto=tcp")
-		    ))
-		pmap->pm_prot = IPPROTO_TCP;
-
-	if (!nfs_gethostbyname(hostname, &mnt_server.saddr)) {
-		nfs_error(_("%s: DNS resolution of '%s' failed"),
-				progname, hostname);
-		goto out;
-	}
-
-	if (!nfs_call_umount(&mnt_server, &dirname)) {
-		nfs_error(_("%s: Server failed to unmount '%s'"),
-				progname, spec);
-		result = EX_FAIL;
-		goto out;
-	}
-
-	result = EX_SUCCESS;
-
-out:
 	free(hostname);
 	free(dirname);
 	return result;
@@ -350,16 +352,16 @@ int nfsumount(int argc, char *argv[])
 	ret = 0;
 	if (mc) {
 		if (!lazy && strcmp(mc->m.mnt_type, "nfs4") != 0)
-			/* We ignore the error from do_nfs_umount23.
+			/* We ignore the error from nfs_umount23.
 			 * If the actual umount succeeds (in del_mtab),
 			 * we don't want to signal an error, as that
 			 * could cause /sbin/mount to retry!
 			 */
-			do_nfs_umount23(mc->m.mnt_fsname, mc->m.mnt_opts);
-		ret = del_mtab(mc->m.mnt_fsname, mc->m.mnt_dir);
+			nfs_umount23(mc->m.mnt_fsname, mc->m.mnt_opts);
+		ret = del_mtab(mc->m.mnt_fsname, mc->m.mnt_dir) ?: ret;
 	} else if (*spec != '/') {
 		if (!lazy)
-			ret = do_nfs_umount23(spec, "tcp,v3");
+			ret = nfs_umount23(spec, "tcp,v3");
 	} else
 		ret = del_mtab(NULL, spec);
 
